#! /bin/sh
#
# patchaic.sh release 1.20
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/
#
# This script modifies BeOS R4.5.1/4.5.2 aic78xx driver
# for use with some Adaptec SCSI adapters.
# Runnable on UNIX in making an installation floppy
#
# 1.00 first release
# 1.10 AHA-2930CU/U supported
# 1.20 R4.5.2 CD driver supported, echo detection bug fixed
#
# PCI device ID / vendor ID / subsystem ID / subsystem vendor ID
# AHA-2930CU (AIC-7859): 3860 9004 3869 9004
# AHA-2930U  (AIC-7880): 8878 9004 7888 9004
# AHA-2930U2 (AIC-7890): 0011 9005 0181 9005
# (from FreeBSD 3.x /sys/pci/ahc_pci.c)

[ "x$DIR" = x ] && DIR=/system/add-ons/kernel/busses/scsi
FILE=aic78xx
# R4.5+update1
OLDSUM451=801274421
# R4.5+update2
OLDSUM452=786147219
# R4.5.2 CD
OLDSUM452CD=2930185480

# look for useful echo command to make this script run on UNIX

# System V echo has octal format of \0xxx but we don't use it
# because it is hard to coexist with other echo's

if [ `/bin/echo \\\\100` = @ ]
then
	ECHO="/bin/echo -n"
elif [ -x /usr/local/bin/echo ] && [ `/usr/local/bin/echo \\\\100` = @ ]
then
	ECHO=/usr/local/bin/echo -n	# expecting GNU shell-utils
elif which -s printf
then
	ECHO=printf	# 4.4BSD /usr/bin/printf
else
	echo "No useful echo command."
	exit 1
fi

# check parameters

if [ $# -ne 1 ]
then
	echo "Usage: $0 {aha2930cu|aha2930u|aha2930u2}"
	exit 1
fi

case $1 in
aha2930cu)
	ID=\\140\\070
	OFS=117114		# 0x1c97a
	;;
aha2930u)
	ID=\\170\\210
	OFS=117114		# 0x1c97a
	;;
aha2930u2)
	ID=\\021\\000
	OFS=117146		# 0x1c99a
	;;
*)
	echo "Unknown chipset: $1"
	exit 1
esac

# check original driver

if [ ! -f $DIR/$FILE ]
then
	echo "Original driver $DIR/$FILE not present."
	exit 1
fi

cd $DIR
OLDSUM=`cksum $FILE | cut -d ' ' -f 1`
case "$OLDSUM" in
$OLDSUM451)
	echo "R4.5.1 aic78xx driver detected."
	;;
$OLDSUM452|$OLDSUM452CD)
	echo "R4.5.2 aic78xx driver detected."
	;;
*)
	echo "Unsupported version of aic78xx driver or patch is already applied."
	exit 1
esac

# back up original driver

[ "x$NOBACKUP" = x ] && gzip -c $FILE > $FILE.gz

# apply patch to driver

chmod u+w $FILE
$ECHO $ID | dd of=$FILE bs=1 seek=$OFS conv=notrunc

# checking new driver is omitted because there are many patterns
# depending version and target

echo "All done."
[ "x$NOBACKUP" = x ] && echo "Original driver is in $FILE.gz"
exit 0
